local registry = LibStub('AceConfigRegistry-3.0')

local function RestoreDefaults(info)
	DeathClock:RestoreFrameDefaults(info.arg)
	DeathClock:ApplySettings()
end

local function GetFontList()
	return AceGUIWidgetLSMlists.font
end

local function GetFrameValue(info)
	return DeathClock.db.profile.clocks[info.arg[1]][info.arg[2]]
end

local function SetFrameValue(info, value)
	DeathClock.db.profile.clocks[info.arg[1]][info.arg[2]] = value
	DeathClock:ApplySettings()
	if info.arg[2] == "show" then
		registry:NotifyChange("Death Clock")
	end
end


local function GetValue(info)
	return DeathClock.db.profile[info.arg]
end

local function SetValue(info, value)
	DeathClock.db.profile[info.arg] = value
	DeathClock:ApplySettings()
end

local function GetColorValue(info)
	local color = DeathClock.db.profile.clocks[info.arg[1]][info.arg[2]]
	return color.r, color.g, color.b, color.a
end

local function SetColorValue(info, r, g, b, a)
	DeathClock.db.profile.clocks[info.arg[1]][info.arg[2]] = {["r"] = r, ["g"] = g, ["b"] = b, ["a"] = a}
	DeathClock:ApplySettings()
end

function GetClockOptionsTable(unitText,order,highlightName)
	local options = {
		name = unitText,
		type = 'group',
		["order"] = order,
		set = SetFrameValue,
		get = GetFrameValue,
		args = {
			["Enable"] = {
				name = "Show",
				type = 'toggle',
				order = 100,
				width = 'half',
				arg = {unitText,'show'},
				disabled = false,
			},
			["Lock"] = {
				name = "Lock",
				type = 'toggle',
				order = 125,
				width = 'half',
				arg = {unitText,'lock'},
				disabled = false,
			},
			["BREAK0"] = {
				name = "",
				type = 'description',
				order = 140,
			},
			["Font Size"] = {
				name = "Font Size",
				type = 'range',
				min = 4, max = 20, step = 1,
				order = 150,
				arg = {unitText,'size'},
			},
			["BREAK1"] = {
				name = "",
				type = 'description',
				order = 160,
			},
			["Font"] = {
				name = "Font",
				type = 'select',
				dialogControl = 'LSM30_Font',
				values = GetFontList,
				order = 200,
				arg = {unitText,'font'},
			},
			["BREAK2"] = {
				name = "",
				type = 'description',
				order = 250,
			},
			["Outline"] = {
				name = "Outline",
				type = 'select',
				desc = "Set font outline.",
				values = {[''] = NONE, OUTLINE = VOICE_CHAT_NORMAL, THICKOUTLINE = "Thick"},
				order = 325,
				arg = {unitText,'outline'},
			},
			["BREAK3"] = {
				name = "",
				type = 'description',
				order = 340,
			},
			["FColor"] = {
				name = "Text Color",
				type = 'color',
				hasAlpha = true,
				set = SetColorValue,
				get = GetColorValue,
				order = 350,
				arg = {unitText,'foreground'},
			},
			["BREAK4"] = {
				name = "",
				type = 'description',
				order = 400,
			},
			["BColor"] = {
				name = "Background",
				type = 'color',
				hasAlpha = true,
				set = SetColorValue,
				get = GetColorValue,
				order = 450,
				arg = {unitText,'background'},
			},
			["BREAK5"] = {
				name = " \n \n \n ",
				type = 'description',
				order = 525,
			},
			["Reset"] = {
				name = "Restore Defaults",
				func = RestoreDefaults,
				type = 'execute',
				order = 550,
				arg = unitText,
			},
		}
	}
	if GetFrameValue({arg = {unitText,"show"}}) then
		options.name = "|cff00ccff" .. unitText.."|r"
	else
		options.name = "|cffff0000" .. unitText.."|r"
		options.order = order + 500
	end
	
	return options
end

function GetClocksOptionsTable()
	local options = {
		name = '',
		type = 'group',
		set = SetValue,
		get = GetValue,
		handler = DeathClock,
		args = {},
	}
	local ids = {}
	for unitText,unitID in pairs(DeathClock.unitIDDisplayText) do
		tinsert(ids,unitText)
	end
	sort(ids)
	for i, unitText in ipairs(ids) do
		options.args[unitText] = GetClockOptionsTable(unitText,i)
	end
	
	return options
end

function GetGeneralOptionsTable()
	return {
		name = 'Death Clock',
		type = 'group',
		set = SetValue,
		get = GetValue,
		handler = DeathClock,
		args = {
			["Enable"] = {
				name = "Enable",
				type = 'toggle',
				order = 100,
				width = 'full',
				arg = 'enabled',
				disabled = false,
			},
			["UPDATE"] = {
				name = "Seconds Between Updates",
				type = 'range',
				width = 'double',
				min = .1, max = 2, step = .1,
				order = 150,
				arg = 'timeBetweenUpdates',
			},
			["BREAK1"] = {
				name = "",
				type = 'description',
				order = 160,
			},
			["Stability"] = {
				name = "Reactivity vs. Stability",
				type = 'range',
				width = 'double',
				min = 3, max = 30, step = 1,
				order = 200,
				arg = 'clockTimeSpan',
			},
		
		
		},	
	}

end

function GetListOptionsTable()
	return {
		name = '',
		type = 'group',
		set = SetValue,
		get = GetValue,
		handler = DeathClock,
		args = {
			["Enable"] = {
				name = "Show",
				type = 'toggle',
				order = 100,
				width = 'half',
				arg = 'listEnabled',
				disabled = false,
			},
			["Lock"] = {
				name = "Lock",
				type = 'toggle',
				order = 150,
				width = 'half',
				arg = 'listLocked',
				disabled = false,
			},
			["Group"] = {
				name = "Lock",
				type = 'toggle',
				order = 150,
				width = 'half',
				arg = 'listGroup',
				disabled = false,
			},
			["UPDATE"] = {
				name = "Seconds Between Updates",
				type = 'range',
				width = 'double',
				min = .1, max = 2, step = .1,
				order = 150,
				arg = 'timeBetweenUpdates',
			},
			["BREAK1"] = {
				name = "",
				type = 'description',
				order = 160,
			},
			["Stability"] = {
				name = "Reactivity vs. Stability",
				type = 'range',
				width = 'double',
				min = 3, max = 30, step = 1,
				order = 200,
				arg = 'clockTimeSpan',
			},
			["KEY1"] = {
				name = "",
				type = 'select',
				disabled = function() return not AllPlates.profile.enableOutOfCombatSorting and not AllPlates.profile.enableInCombatSorting end,
				desc = "",
				values = {['name']= "Name",['raidicon']="Raid Icon",['groupSize']="Group Size",['health']="Health",['timeTillDeath']="Time Till Death"},
				order = 50,
				arg = {1,2},
			},
		},	
	}
end


function GetDefaults()
	local defaults = {
		profile = {
			clocks = {},
			enabled = true,
			timeBetweenUpdates = .4,
			clockTimeSpan = 8,
			
			listX = 500,
			listY = 500,
			listP1 = "TOPLEFT",
			listP2 = "CENTER",
			
			listScale = 1,
			listWidth = 220,
			listHeight = 200,
			listEnabled = true,
			listLocked = false,
			
		}
	}
	
	
	for unitText,unitID in pairs(DeathClock.unitIDDisplayText) do
		defaults.profile.clocks[unitText] = DeathClock:GetTableDefaults(unitText)
	end
	
	return defaults
end

function DeathClock:ProfileChanged()
	for _,frame in pairs(self.frames) do
		frame.anchorSecure = nil
	end
	-- Call it once ahead of time to ensure all default settings are in place.
	GetClocksOptionsTable()
	self:ApplySettings()
end

function DeathClock:RegisterOptions()
 	self.db = LibStub('AceDB-3.0'):New('DeathClockDB', GetDefaults(), 'Default')
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	self:RegisterMedia()
	
	local profileOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	local dialog = LibStub('AceConfigDialog-3.0')
	registry:RegisterOptionsTable("Death Clock", {name = "",type = 'group',desc = "",args = {}})
	local top = dialog:AddToBlizOptions("Death Clock", "Death Clock")
	top:SetScript('OnShow', function(self)
		InterfaceOptionsFrame_OpenToCategory(DeathClock.optionsDialog)
	end)
	
	registry:RegisterOptionsTable('Death Clock - General', GetGeneralOptionsTable)
	DeathClock.optionsDialog = dialog:AddToBlizOptions('Death Clock - General', 'General',"Death Clock")
	
	registry:RegisterOptionsTable('Death Clock - Clocks', GetClocksOptionsTable)
	dialog:AddToBlizOptions('Death Clock - Clocks', 'Clock Frames',"Death Clock")

	registry:RegisterOptionsTable("Death Clock Profile", profileOptions)
	dialog:AddToBlizOptions("Death Clock Profile", "Profile", "Death Clock")
	
	self:ProfileChanged()
end

function DeathClock:RegisterMedia()
	local media = LibStub("LibSharedMedia-3.0");
	media:Register("font", "ABF",					[[Interface\Addons\death-clock\fonts\ABF.ttf]])
	media:Register("font", "Accidental Presidency",	[[Interface\Addons\death-clock\fonts\Accidental Presidency.ttf]])
	media:Register("font", "Adventure",				[[Interface\Addons\death-clock\fonts\Adventure.ttf]])
	media:Register("font", "AnonymousPro (Bold)",	[[Interface\Addons\death-clock\fonts\AnonymousPro-Bold.ttf]])
	media:Register("font", "Avqest",				[[Interface\Addons\death-clock\fonts\Avqest.ttf]])
	media:Register("font", "Bazooka",				[[Interface\Addons\death-clock\fonts\Bazooka.ttf]])
	media:Register("font", "BigNoodleTitling",		[[Interface\Addons\death-clock\fonts\BigNoodleTitling.ttf]])
	media:Register("font", "BigNoodleTitling-Oblique",[[Interface\Addons\death-clock\fonts\BigNoodleTitling-Oblique.ttf]])
	media:Register("font", "BlackChancery",			[[Interface\Addons\death-clock\fonts\BlackChancery.ttf]])
	media:Register("font", "CRYSRG__",				[[Interface\Addons\death-clock\fonts\CRYSRG__.ttf]])
	media:Register("font", "DejaVuSansMono",		[[Interface\Addons\death-clock\fonts\DejaVuSansMono.ttf]])
	media:Register("font", "DorisPP",				[[Interface\Addons\death-clock\fonts\DORISPP.ttf]])
	media:Register("font", "Emblem",				[[Interface\Addons\death-clock\fonts\Emblem.ttf]])
	media:Register("font", "Enigma__2",				[[Interface\Addons\death-clock\fonts\Enigma__2.ttf]])
	media:Register("font", "FSEX300",				[[Interface\Addons\death-clock\fonts\FSEX300.ttf]])
	media:Register("font", "JAi_____",				[[Interface\Addons\death-clock\fonts\JAi_____.ttf]])
	media:Register("font", "LiberationSans Regular",[[Interface\Addons\death-clock\fonts\LiberationSans-Regular.ttf]])
	media:Register("font", "MONOFONT",				[[Interface\Addons\death-clock\fonts\MONOFONT.ttf]])
	media:Register("font", "Movie Poster (Bold)",	[[Interface\Addons\death-clock\fonts\Movie_Poster-Bold.ttf]])
	media:Register("font", "Porky",					[[Interface\Addons\death-clock\fonts\Porky.ttf]])
	media:Register("font", "rm_midse",				[[Interface\Addons\death-clock\fonts\rm_midse.ttf]])
	media:Register("font", "Tangerin",				[[Interface\Addons\death-clock\fonts\Tangerin.ttf]])
	media:Register("font", "Tw_Cen_MT_Bold",		[[Interface\Addons\death-clock\fonts\Tw_Cen_MT_Bold.ttf]])
	media:Register("font", "Ultima Campagnoli",		[[Interface\Addons\death-clock\fonts\Ultima_Campagnoli.ttf]])
	media:Register("font", "VeraSe",				[[Interface\Addons\death-clock\fonts\VeraSe.ttf]])
	media:Register("font", "WHITRABT",				[[Interface\Addons\death-clock\fonts\WHITRABT.ttf]])
	media:Register("font", "Yellow Jacket",			[[Interface\Addons\death-clock\fonts\Yellowjacket.ttf]])
end
