local LubStub = LibStub
local DeathClock = LibStub('AceAddon-3.0'):NewAddon('DeathClock', 'AceTimer-3.0', 'AceEvent-3.0', 'AceConsole-3.0')
_G.DeathClock = DeathClock
local debug = false
local updateTime
local lastUpdateTime = GetTime()

local function FormatTime(value)
	if not value or value > 604800 or value <= 0 then
		return ""
	elseif value < 10 then
		return format('%.1fs',value);
	elseif value < 60 then
		return format('%ds',value);
	elseif value < 3600 then
		return format('%dm%02ds', 1/60 * value, value % 60);
	elseif value < 86400 then
		return format('%dh%02dm', 1/3600 * value, 1/60 * (value % 3600));
	else
		return format('%dd%02dh', 1/86400 * value, 1/3600 * (value % 86400));
	end
end

local function ClockUpdate(unit,health) 
	if not unit.clocks then
		unit.updateI = 1
		unit.clocks = {}
		for i = 1,floor(DeathClock.db.profile.clockTimeSpan/DeathClock.db.profile.timeBetweenUpdates) do
			unit.clocks[i] = {initHealth = health, initTime = updateTime}
		end
	else
		unit.clocks[unit.updateI] = {initHealth = health, initTime = updateTime}
		if unit.updateI >= floor(DeathClock.db.profile.clockTimeSpan/DeathClock.db.profile.timeBetweenUpdates) then
			unit.updateI = 1
		else
			unit.updateI = unit.updateI + 1
		end
	end
	
	local deltaHealth = unit.clocks[unit.updateI].initHealth - health
	local deltaTime = updateTime - unit.clocks[unit.updateI].initTime
	if deltaHealth > 0 and deltaTime > 0 then
		unit.time = health * deltaTime / deltaHealth
		unit.text = FormatTime(unit.time)
	else	
		unit.time = 0
		unit.text = ""
	end
end

function DeathClock:OnInitialize()
	self.units = {}
	self.frames = {}
	self:RegisterOptions()
	self:RegisterChatCommand("dc", "SlashCommand")
	self:RegisterChatCommand("deathclock", "SlashCommand")
end

function DeathClock:UpdateTimes()
	for _,unitID in pairs(DeathClock.unitIDDisplayText) do
		local guid = UnitGUID(unitID)
		if guid then
			if not self.units[guid] then
				self.units[guid] = {updated = updateTime - 1}
			end
			if self.units[guid].updated < updateTime then
				ClockUpdate(self.units[guid], UnitHealth(unitID))
				self.units[guid].updated = updateTime
				self.units[guid].unitID = unitID
			end
		end
	end
end

function Update()
	updateTime = GetTime()
	if DeathClock.db.profile.enabled and lastUpdateTime + DeathClock.db.profile.timeBetweenUpdates <= updateTime then
		DeathClock:UpdateTimes()
		DeathClock:UpdateClockFrames()
		lastUpdateTime = updateTime
	end
end

function DeathClock:ApplySettings()
	for unitID,frame in pairs(self.frames) do
		self:ApplyFrameSettings( frame)
	end
	
	if not DeathClock.db.profile.enabled then
		for unitID,frame in pairs(self.frames) do
			frame:Hide()
		end
	end
end

function DeathClock:SlashCommand(info)
	if DeathClock.optionsDialog then
		if InterfaceOptionsFrame:IsVisible() then
			InterfaceOptionsFrame:Hide()
		else
			InterfaceOptionsFrame_OpenToCategory(DeathClock.optionsDialog) 
		end
	end
end



local DeathClock_Events = {}
local eventFrame = CreateFrame("Frame", nil, WorldFrame)

local function DoEvent(self, event, ...)
	if DeathClock_Events[event] then
		DeathClock_Events[event](event, ...)
	end
end

eventFrame:SetFrameStrata("TOOLTIP") 
eventFrame:SetScript("OnEvent", DoEvent)
eventFrame:SetScript("OnUpdate",Update)


function DeathClock_Events:PLAYER_REGEN_ENABLED()
	wipe(self.units)
	DeathClock.inCombat = false
end

function DeathClock_Events:PLAYER_REGEN_DISABLED()
	DeathClock.inCombat = true
end

for eventname in pairs(DeathClock_Events) do 
	eventFrame:RegisterEvent(eventname) 
end